; new function for phase 3
function get_hotpix_2d,image,frame_id
  if ~ defined(frame_id) then frame_id=1
  filtered_image=filter_image(image[*,*,frame_id],median=5)
  hotpix_2d=crfind(image[*,*,frame_id]-filtered_image,gain=6,read_noise=20)*(image[*,*,frame_id]-filtered_image(image))
; was read_noise=10
; smaller read_noise setting (<20) sometimes causes all-filtered-out situation, which is not intended.
  return,hotpix_2d
end

function calc_hotpix_scale_fac,image,hotpix_2d,frame_id,hotpix_max

; switch to select warm/cold mode
  if NIR() && !IRC_SPECRED_PROCESSOPTION.warm_mode EQ 0B then $
     image_hotpix=get_hotpix_2d(image,frame_id) $
  else $
     image_hotpix=image[*,*,frame_id]-median(image[*,*,frame_id],5)

; new for phase-3
  if (NIR() && !IRC_SPECRED_PROCESSOPTION.warm_mode EQ 1B) && $
     (!IRC_SPECRED_OBSPARAM.slit_mode EQ 1 || !IRC_SPECRED_OBSPARAM.Np_mode) then begin
     image_hotpix_cut=image_hotpix[1:120,1:400]
     hotpix_2d_cut=hotpix_2d[1:120,1:400]
  endif else begin
     image_hotpix_cut=image_hotpix
     hotpix_2d_cut=hotpix_2d
  endelse
; end of 'new for phase-3'

  image_hotpix_1d=reform(reform(image_hotpix_cut,1,n_elements(image_hotpix_cut)))

  hotpix_1d=reform(reform(hotpix_2d_cut,1,n_elements(hotpix_2d_cut)))

  nonnan=where(finite(hotpix_1d),count)

  image_hotpix_1d_nonnan=image_hotpix_1d[nonnan]
  hotpix_1d_nonnan=hotpix_1d[nonnan]

; tune hotpix_min according to the phase-2 or -3
  if !IRC_SPECRED_PROCESSOPTION.warm_mode then hotpix_min=10. else hotpix_min=100.
  good=where( $
       hotpix_1d_nonnan GT hotpix_min AND $
       image_hotpix_1d_nonnan GT hotpix_min AND $
       image_hotpix_1d_nonnan LT hotpix_max AND $
       image_hotpix_1d_nonnan/image_hotpix_1d_nonnan GT 0.5 AND $
       image_hotpix_1d_nonnan/image_hotpix_1d_nonnan LT 2.0, $
       count)
  
  if count GT 10 then begin
;     plot,image_hotpix_1d_nonnan[good],image_hotpix_1d_nonnan[good]/hotpix_1d_nonnan[good],psym=5,xrange=[0,200],yrange=[0,2];,color=!irc_specred_color.red
;     wait,1
     return,median(image_hotpix_1d_nonnan[good]/hotpix_1d_nonnan[good])
  endif else begin
     return,1.
  endelse
end

pro do_hotpixsub,refimage,refimage_short,specimage,specimage_short, $
                 refdark_ds_long,refdark_ds_short,superdark, $
                 refimage_hs,specimage_hs,hotpix=hotpix

  if NIR() && !IRC_SPECRED_PROCESSOPTION.warm_mode EQ 0B then begin
                                ; for NIR, do nothing...
     refimage_hs=refimage
     specimage_hs=specimage
     return
  endif

  common gui,logger

  if defined(logger) then logger->append,['---','Hotpixel correcting...']

; scale_fac=0.6107; original value
; scale_fac=0.7107; tunied value for FOV L7

  if !IRC_SPECRED_PROCESSOPTION.point_remover_mode EQ 1B then begin
     !IRC_SPECRED_PROCESSOPTION.point_remover_mode=0B 
     irc_imcombine_n,refdark_ds_long,refdark_ds_short,refdark_ds_combined,/no_shift,comb_mode='median' ; since Num(refdarks) > 2 in phase-3, we set comb_mode='median'
     !IRC_SPECRED_PROCESSOPTION.point_remover_mode=1B
  endif else begin
     irc_imcombine_n,refdark_ds_long,refdark_ds_short,refdark_ds_combined,/no_shift,comb_mode='median' ; since Num(refdarks) > 2 in phase-3, we set comb_mode='median'
  endelse

  if !IRC_SPECRED_PROCESSOPTION.warm_mode then $
;    hotpix_2d=refdark_ds_combined[*,*,1]-sigma_filter(refdark_ds_combined[*,*,1],radius=1,/all,/iter,n_sigma=3.) $
;    hotpix_2d=refdark_ds_combined[*,*,1]-sigma_filter(refdark_ds_combined[*,*,1],radius=2,/all,/iter,n_sigma=3.) $
               hotpix_2d=get_hotpix_2d(refdark_ds_combined) $ ; use new hot-pixel detection method for phase-3
;     hotpix_2d=crfind(refdark_ds_combined[*,*,1]-filter_image(refdark_ds_combined[*,*,1],median=5),gain=6,read_noise=10,/ver)*(refdark_ds_combined[*,*,1]-filter_image(refdark_ds_combined[*,*,1],median=5)) $
  else $
     hotpix_2d=refdark_ds_combined[*,*,1]-filter_image(refdark_ds_combined[*,*,1],median=5)

;  atv,hotpix_2d,/b

  hotpix_2d_org=hotpix_2d

  if !IRC_SPECRED_PROCESSOPTION.warm_mode EQ 0B then $
     hotpix_max=100 $
  else $
     hotpix_max=1000             ; was 50 bofore phase-3 --- incrasing hotpix_max means we subtract even hotter hotpixels than ever.

 if !IRC_SPECRED_PROCESSOPTION.warm_mode EQ 0B then $
     hotpix_min=-100 $
  else $
     hotpix_min=-100

  hottest_pix=where(hotpix_2d GT hotpix_max,count)
  if count GT 0 then hotpix_2d[hottest_pix]=!values.f_nan
  print,'Information (do_hotpixsub): # of hotpixels masked: '+strim(count)

;  coldest_pix=where(hotpix_2d LT hotpix_min,count)
;  if count GT 0 then hotpix_2d[coldest_pix]=!values.f_nan
;  print,'Information (do_hotpixsub): # of coldpixels masked: '+strim(count)

  refimage_hs=refimage
  specimage_hs=specimage

; window,xsize=800,ysize=500

  hotpix_max=10000
  s=size(refimage) & if s[0] EQ 3 then n_frame=s[3] else n_frame=1
  for frame_id=0,n_frame-1 do begin
 ;   if !IRC_SPECRED_PROCESSOPTION.warm_mode then scale_fac=1.0 else $
     scale_fac=calc_hotpix_scale_fac(refimage,hotpix_2d_org,frame_id,hotpix_max)
     refimage_hs_tmp=refimage[*,*,frame_id]-hotpix_2d*scale_fac
     refimage_hs[*,*,frame_id]=refimage_hs_tmp
  endfor
  
  s=size(specimage) & if s[0] EQ 3 then n_frame=s[3] else n_frame=1
  for frame_id=0,n_frame-1 do begin
;    if !IRC_SPECRED_PROCESSOPTION.warm_mode then scale_fac=1.0 else $
     scale_fac=calc_hotpix_scale_fac(specimage,hotpix_2d_org,frame_id,hotpix_max)
;    print,scale_fac
     specimage_hs_tmp=specimage[*,*,frame_id]-hotpix_2d*scale_fac
     specimage_hs[*,*,frame_id]=specimage_hs_tmp
  endfor

  hotpix=refdark_ds_combined
  hotpix[*,*,1]=hotpix_2d
  hotpix[*,*,0]=hotpix_2d

end
